from typing import List, Tuple
from pandas import DataFrame, Series
from numbers import Real


class QuantitativeFrame:
    def __init__(self, data: List[Real], dataframe: DataFrame, interval: int) -> None:
        self.data = data
        self.dataframe = dataframe
        self.interval = interval
        self.__median_index: int = None
        self.__trend_index: int = None

    def __find_median_row(self) -> Tuple[int, Series]:
        n_2 = len(self.data) / 2
        return next((i, row) for i, row in self.dataframe.iterrows() if row["Ni"] > n_2)

    def __find_trend_row(self) -> Tuple[int, Series]:
        index = self.__trend_index = self.dataframe["ni"].argmax()
        return (index, self.dataframe.iloc[self.__trend_index])

    def arithmetic_mean(self) -> Real:
        return sum(self.dataframe["mi x ni"]) / len(self.data)

    def median(self) -> Real:
        (self.__median_index, median_row) = self.__find_median_row()
        Fi = self.dataframe.iloc[self.__median_index - 1]["Ni"]
        fi = median_row["ni"]
        Linf = median_row["Clase"][0]
        A = self.interval
        n_2 = len(self.data) / 2

        return Linf + A * ((n_2 - Fi) / fi)

    def trend(self) -> Real:
        (index, row) = self.__find_trend_row()
        Linf = row["Clase"][0]
        A = self.interval
        fi = row["ni"]
        Fi = self.dataframe.iloc[index - 1]["ni"]
        fi_1 = self.dataframe.iloc[index + 1]["ni"]

        return Linf + A * (fi - Fi) / ((fi - Fi) + (fi - fi_1))

    def median_row(self) -> DataFrame:
        if self.__median_index is None:
            (self.__median_index, _) = self.__find_median_row()
        return self.dataframe.iloc[[self.__median_index]]

    def trend_row(self) -> DataFrame:
        (index, _) = self.__find_trend_row()
        return self.dataframe.iloc[[index]]



from typing import Any, List, Dict, Union
from pandas import DataFrame, Series


def categorical_table(data: Union[List, Dict]) -> DataFrame:
    mapped_data = data
    if isinstance(data, list):
        mapped_data = map_categorical_data(data)

    cumulative_absolute = Series(mapped_data.values()).cumsum()
    relative_frecuency = [v / sum(mapped_data.values()) for v in mapped_data.values()]
    cumulative_relative = Series(relative_frecuency).cumsum()
    percentage_frequency = [i * 100 for i in relative_frecuency]

    return DataFrame(
        {
            "ni": mapped_data.values(),
            "Ni": cumulative_absolute.tolist(),
            "hi": relative_frecuency,
            "Hi": cumulative_relative.tolist(),
            "%": percentage_frequency,
            "% acum": Series(percentage_frequency).cumsum().tolist(),
        },
        index=mapped_data.keys(),
    ).rename_axis("Clase", axis=1)


def map_categorical_data(data: List[Any]) -> Dict[Any, int]:
    mapped = dict()
    for element in data:
        mapped[element] = mapped.get(element, 0) + 1
    return mapped



from typing import List, Dict, Tuple
from pandas import DataFrame, Series
from enum import Enum, auto
from numbers import Real
from numpy import arange
from math import ceil, log10
from marco.quantitative_frame import QuantitativeFrame


class QuantitativeType(Enum):
    DISCRETE = auto()
    CONTINUOUS = auto()


QuantitativeClass = List[Tuple[Real, Real]]


def quantitative_table(
    data: List[Real], type=QuantitativeType.CONTINUOUS
) -> QuantitativeFrame:
    Ro = range_set(data)
    class_n = class_amount(len(data))
    fixed_data = apply_range_fix(data, Ro)
    interval_value = interval(Ro, class_n)
    q_class = quantitative_class(fixed_data, interval_value, class_n)
    mi = class_mark(q_class)
    ni = quantitative_absolute_frecuency(sorted(data), q_class)
    mi_x_ni = [a * b for a, b in zip(mi, ni.values())]

    return QuantitativeFrame(
        data,
        DataFrame(
            {
                "Clase": q_class,
                "mi": mi,
                "ni": ni.values(),
                "Ni": Series(ni.values()).cumsum().tolist(),
                "mi x ni": mi_x_ni,
            },
        ),
        interval_value,
    )


def range_set(data: List[Real]) -> Real:
    return max(data) - min(data)


def class_amount(data_amount: int) -> int:
    return ceil(1 + 3.3 * log10(data_amount))


def interval(range_interval: Real, class_amount: int) -> int:
    return ceil(range_interval / class_amount)


def quantitative_class(
    data: List[Real], interval: int, class_amount: int
) -> QuantitativeClass:
    tuples_list = []
    lower_limit = data[0]
    for _ in range(class_amount):
        upper_limit = lower_limit + interval
        tuples_list.append((lower_limit, upper_limit))
        lower_limit += interval

    return tuples_list


def class_mark(quantitative_class: QuantitativeClass) -> List[Real]:
    return [(q[0] + q[1]) / 2 for q in quantitative_class]


def fix_range(class_amount: int, interval: int, range_interval: Real) -> Real:
    return class_amount * interval - range_interval


def apply_range_fix(data: List[Real], range_interval: Real) -> List[Real]:
    sorted_data = sorted(data)
    class_n = class_amount(len(data))
    interval_value = interval(range_interval, class_n)
    range_fix = fix_range(class_n, interval_value, range_interval) / 2
    sorted_data[0] -= range_fix
    sorted_data[-1] += range_fix
    return sorted_data


def quantitative_absolute_frecuency(
    data: List[Real], q_class: QuantitativeClass
) -> Dict[str, int]:
    mapped = dict()
    i = j = 0
    while i < len(data):
        if data[i] in arange(q_class[j][0], q_class[j][1], 0.5):
            mapped[str(q_class[j])] = mapped.get(str(q_class[j]), 0) + 1
            i += 1
        else:
            if j < len(q_class) - 1:
                j += 1
    return mapped



from setuptools import setup, find_packages

setup(
    name="marco",
    packages=find_packages(include=['marco'], exclude=['examples']),
    version="0.1.1",
    license="MIT",
    description="Statistics class automation",
    author="Andres Galvan",
    author_email="adfelipegalvan@gmail.com",
    url="https://github.com/afgalvan/marco",
    download_url="https://github.com/afgalvan/marco/archive/v_01.tar.gz",
    keywords=["statistics", "pandas", "numpy"],
    install_requires=[
        "pandas",
        "numpy"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
)
